from unleashed_chat.openai_client import create_openai_client
from unleashed_chat.config_handler import request_api_key, request_model, get_api_key
from unleashed_chat.session import run_session
from unleashed_chat.cli import parse_arguments

# Gets config information and start the session.
def main():
  args = parse_arguments()

  if args.set_api_key:
    request_api_key()
    return
  elif args.set_model:
    request_model()
    return
  
  api_key = get_api_key()
  if api_key is None:
      return

  openai_client = create_openai_client(api_key)

  run_session(openai_client)

if __name__ == "__main__":
  main()


from openai import OpenAI, BadRequestError
from unleashed_chat.config_handler import request_model, get_model

# Creates an OpenAI client with the provided API key
# using the Unleashed.chat API endpoint.
def create_openai_client(api_key):
  return OpenAI(
    base_url='https://unleashed.chat/api/v1',
    api_key=api_key
  )

# Sends a prompt to the OpenAI API and returns the response.
def get_response(openai_client, prompt, model, memory=[]):
  continue_conversation = True
  if(prompt == None):
    return None, continue_conversation
  if(prompt == ""):
    return "Prompt cannot be empty.", continue_conversation
  if(prompt.lower() == "/help"):
    return "Available commands:\n/exit (end the session)\n/reset (reset the memory)\n/models (view available models)\n/model (check the currently used model)\n/setmodel (prompts you to set the model to use)\n/help (view available commands)", continue_conversation
  elif(prompt.lower() == "/exit"):
    continue_conversation = False
    return "Goodbye!", continue_conversation
  elif(prompt.lower() == "/reset"):
    memory.clear()
    return "Memory has been reset! You are now in a new session.", continue_conversation
  elif(prompt.lower() == "/models"):
    response = get_models_list(openai_client)
    return response, continue_conversation
  elif(prompt.lower() == "/setmodel"):
    print("\n" + get_models_list(openai_client) + "\n")
    request_model()
    return "Model has been set!", continue_conversation
  elif(prompt.lower() == "/model"):
    model = get_model()
    return "You are using model: " + model, continue_conversation
  
  memory.append({"role": "user", "content": prompt})

  response_stream = None
  try:
    response_stream = openai_client.chat.completions.create(
      messages=memory,
      model=model,
      stream=True,
    )
  except BadRequestError as e:
    print(f"An error occurred: {e}")
    return str(e), continue_conversation

  response = ""
  for chunk in response_stream:
    response += chunk.choices[0].delta.content

  memory.append({"role": "assistant", "content": response})
  
  return response, continue_conversation

# Return a string with the list of available models.
def get_models_list(openai_client):
  response = "Available models:\n"
  response += "\n".join(model.id for model in openai_client.models.list())
  return response


import argparse

def parse_arguments():
  parser = argparse.ArgumentParser(
    prog="Unleashed Chat CLI",
    description='Unleashed Chat CLI',
    epilog="A CLI wrapper for Unleashed.chat's chatbot service."
  )
  parser.add_argument('--set-api-key', action='store_true', help='set your Unleashed API key')
  parser.add_argument('--set-model', action='store_true', help='set the model to use')
  return parser.parse_args()


import configparser

# Gets the configuration file.
def get_config():
    config = configparser.ConfigParser()
    config.read('config.ini')
    return config

# Requests the user to input their Unleashed API key.
def request_api_key():
    api_key = input("Please enter your Unleashed API key: ")

    config = get_config()

    if 'Unleashed' not in config:
        config.add_section('Unleashed')

    config.set('Unleashed', 'api_key', api_key)

    with open('config.ini', 'w') as configfile:
        config.write(configfile)

    print("API key saved.")

# Requests the user to input the model they would like to use.
def request_model():
    model = input("Please enter the model you would like to use: ")

    config = get_config()

    if 'Unleashed' not in config:
        config.add_section('Unleashed')

    config.set('Unleashed', 'model', model)

    with open('config.ini', 'w') as configfile:
        config.write(configfile)

    print("Model saved.")

# Gets the model from the configuration file.
def get_model():
    config = get_config()
    model = None
    try:
        model = config.get('Unleashed', 'model')
    except:
        model = "dolphin-2.2.1-mistral-7b"
    return model

# Gets the API key from the configuration file.
def get_api_key():
    config = get_config()
    try:
        return config.get('Unleashed', 'api_key')
    except configparser.NoSectionError:
        print("No 'Unleashed' section in the configuration file.")
    except configparser.NoOptionError:
        print("No API key found. Starting setup.")


from unleashed_chat.openai_client import get_response
from unleashed_chat.config_handler import get_model
from openai import AuthenticationError

# Runs the chat session.
def run_session(openai_client):
  memory = []
  active_session = True

  print("AI: Hello! How many I assist you today? You can type /help to see available commands.\n")

  while active_session:
    prompt = None
    try:
      prompt = input("You: ")
    except KeyboardInterrupt:
      print("\n\nAI: Goodbye!")
      active_session = False
    try:
      model = get_model()
      response, continue_conversation = get_response(openai_client, prompt, model, memory)
    except AuthenticationError:
      print("\nAI: Invalid API key. Please set your API key using the --set-api-key startup flag and try again.\n")
      return
    if(response != None and response != ""):
      print("\nAI:", response + "\n")
    if(not continue_conversation):
      active_session = False


from setuptools import setup, find_packages

setup(
    name='unleashed-chat',
    version='0.1.0',
    author="Reece Hunter",
    description="A CLI wrapper for Unleashed.chat's chatbot service.",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/reecehunter/unleashed-chat",
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.11',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'openai'
    ],
    entry_points={
        'console_scripts': [
            'unleashed-chat=unleashed_chat.main:main',
        ],
    },
)